;------------------------------------------------------------
; Library TIMERS
; Description: library for delays a time measurement on C64
; Author: Tomas Pribyl - dundera@volny.cz
; Version: 1.0
; Last Revision: 17.9.2000
;
; PAL_Clock  =  985248Hz ;PAL  systems FI2 clock frequency
; NTSC_Clock = 1022727Hz ;NTSC systems FI2 clock frequency

CIA_TALO = $dd04
CIA_TAHI = $dd05
CIA_TBLO = $dd06
CIA_TBHI = $dd07
CIA_CRA  = $dd0e
CIA_CRB  = $dd0f



;------------------------------------------------------------
; Routine MAKEDELAY
; Makes delay 1-65535 miliseconds
;
; Input: miliseconds; X - low, Y - high
; Output: none
; Uses:  TIMER A in CIA2
; Note: works correctly only on PAL systems

Timer1ms = 985-CalDelay         ; number FI2 cycles during 1ms - routine delay
CalDelay = 4+4+4+4+2+4+3+2+2+3  ; program delay - number FI2 cycles per 1ms

MakeDelay = *
         php
         sei
TimDelay lda #<timer1ms    ;4
         sta CIA_TALO      ;4
         lda #>timer1ms    ;4
         sta CIA_TAHI      ;4

         lda #%00011001    ;2
         sta CIA_CRA       ;4

         lda #$01
EndDelay bit CIA_CRA
         bne EndDelay      ;3
         dex               ;2
         cpx #$ff          ;2
         bne TimDelay      ;3
         dey               ;2
         cpy #$ff          ;2
         bne TimDelay      ;3
         plp
         rts               ;6




;------------------------------------------------------------
; Routine STARTTIME
; Starts timer
;
; Input: none
; Output: none
; Uses:  TIMER A and B in CIA2

StartTime
         lda #$ff          ;set maximum range $FFFFFFFF
         sta CIA_TALO
         sta CIA_TAHI
         sta CIA_TBLO
         sta CIA_TBHI

         lda #%01011001    ;TB count TA underflow, OneShot,Start TB
         sta CIA_CRB
         lda #%00010001    ;TA count FI2, Continuous, Start TA
         sta CIA_CRA
         rts




;------------------------------------------------------------
; Routine SAVETIME
; Stops timers, reads timers and counts number of FI2 cycles
; between STARTTIME and SAVETIME
;
; Input: X - zero page pointer for results
; Output: 4 bytes on zeropage (number of FI2 cycles)
; Uses:  TIMER A and B in CIA2

SaveTime lda #%00000000
         sta CIA_CRA
         sta CIA_CRB       ;stop TIMERS

         lda CIA_TALO
         eor #$ff
         sta $00,x         ;time byte0

         lda CIA_TAHI
         eor #$ff
         sta $01,x         ;time byte1

         lda CIA_TBLO
         eor #$ff
         sta $02,x         ;time byte2

         lda CIA_TBHI
         eor #$ff
         sta $03,x         ;time byte3
         rts

